import Flutter
import UIKit

struct ShareItParameters: Decodable {
	enum ShareItType: String, Decodable {
		case text, image, file
	}
	let content: String
	let type: ShareItType
}

public class SwiftShareItPlugin: NSObject, FlutterPlugin {
	
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "share_it", binaryMessenger: registrar.messenger())
    let instance = SwiftShareItPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

	if call.method == "shareItiOS" {
		guard let arguments = call.arguments else {
			result("Empty arguments")
			return
		}
		
		guard let shareItiOSData = try? JSONSerialization.data(withJSONObject: arguments, options: []) else {
			result("Invalid arguments")
			return
		}
		guard let parameters = try? JSONDecoder().decode(ShareItParameters.self, from: shareItiOSData) else {
			result("Invalid decoding of arguments")
			return
		}
		
		activityController(parameters: parameters) {
			result(true)
		}
	}
	
  }
	
	// private
	
	private func activityItems(for parameters: ShareItParameters) -> [Any] {
		switch parameters.type {
		case .text:
			return [parameters.content]
		case .image:
			let url = URL(fileURLWithPath: parameters.content)
			guard let data = try? Data(contentsOf: url) else { return [] }
			guard let image = UIImage(data: data) else { return [] }
			return [image]
		case .file:
			let url = URL(fileURLWithPath: parameters.content)
			guard let data = try? Data(contentsOf: url) else { return [] }
			return [data]
		}
	}
	
	private func activityController(parameters: ShareItParameters, completionHandler: @escaping () -> Void) {
		let activityVC = UIActivityViewController(activityItems: activityItems(for: parameters), applicationActivities: nil)
		UIApplication.shared.keyWindow?.rootViewController?.present(activityVC, animated: true, completion: completionHandler)
	}
}
